
#include "config.h"

//
// Stolen from Python.h
//
#include <stdio.h>
#include <string.h>
#include <errno.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif


#include "pyutil.h"


//
// "Py.newInteger(value)"
//
jobject NewPyInteger(JNIEnv *env, jint value) {
    jclass jpycls = env->FindClass("org/python/core/Py");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jnewint = env->GetStaticMethodID(jpycls, "newInteger",
			"(I)Lorg/python/core/PyInteger;");
    CHECK(jnewint, return NULL, "get Py.newInteger failed");

    jobject jpyint = env->CallStaticObjectMethod(jpycls, jnewint, value);
    CHECK(jpyint, return NULL, "Py.newInteger(...) failed");

    return jpyint;
}


//
// "New PyLong(long value)"
//
jobject NewPyLong(JNIEnv *env, jlong value) {
    jclass jpycls = env->FindClass("org/python/core/PyLong");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/PyLong) failed");

    jmethodID jinit = env->GetMethodID(jpycls, "<init>", "(J)V");
    CHECK(jinit, return NULL, "get PyLong.<init> failed");

    jobject jpylong = env->NewObject(jpycls, jinit, value);
    CHECK(jpylong, return NULL, "new PyLong(...) failed");

    return jpylong;
}



//
// "Py.newString(value)
//
jobject NewPyString(JNIEnv *env, jstring value) {
    jclass jpycls = env->FindClass("org/python/core/Py");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jnewstr = env->GetStaticMethodID(jpycls, "newString",
			"(Ljava/lang/String;)Lorg/python/core/PyString;");
    CHECK(jnewstr, return NULL, "get Py.newString failed");

    jobject jpystr = env->CallStaticObjectMethod(jpycls, jnewstr, value);
    CHECK(jpystr, return NULL, "Py.newString(...) failed");

    return jpystr;
}


//
// "Py.newString(value)
//
jobject NewPyString(JNIEnv *env, const char* value) {
    jclass jpycls = env->FindClass("org/python/core/Py");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jnewstr = env->GetStaticMethodID(jpycls, "newString",
			"(Ljava/lang/String;)Lorg/python/core/PyString;");
    CHECK(jnewstr, return NULL, "get Py.newString failed");

    jstring jstr = env->NewStringUTF(value);
    CHECK(jstr, return NULL, "get jstring failed");

    jobject jpystr = env->CallStaticObjectMethod(jpycls, jnewstr, jstr);
    CHECK(jpystr, return NULL, "Py.newString(...) failed");

    return jpystr;
}



//
// "Py.newFloat(value)
//
jobject NewPyFloat(JNIEnv *env, jdouble value) {
    jclass jpycls = env->FindClass("org/python/core/Py");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jnewfloat = env->GetStaticMethodID(jpycls, "newFloat",
			"(D)Lorg/python/core/PyFloat;");
    CHECK(jnewfloat, return NULL, "get Py.newFloat failed");

    jobject jpyfloat = env->CallStaticObjectMethod(jpycls, jnewfloat, value);
    CHECK(jpyfloat, return NULL, "Py.newFloat(...) failed");

    return jpyfloat;
}




//
// Py.<type>
//
jobject excType(JNIEnv *env, char *type) {
    jclass jcls = env->FindClass("org/python/core/Py");
    CHECK(jcls, return NULL, "FindClass(org/python/core/Py) failed");

    jfieldID jfld = env->GetStaticFieldID(jcls, type, "Lorg/python/core/PyObject;");
    CHECK(jfld, return NULL, "Py.<exctype> failed");

    jobject jobj = env->GetStaticObjectField(jcls, jfld);
    CHECK(jobj, return NULL, "get Py.<exctype> failed");

    return jobj;
}


//
// Py.makeException(<jtype>, <jvalue>)
//
jthrowable PyException(JNIEnv *env, jobject jtype, jobject jvalue) {
    CHECK(jtype, return NULL, "PyException type is null");

    jclass jcls = env->FindClass("org/python/core/Py");
    CHECK(jcls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jinit = env->GetStaticMethodID(jcls, "makeException",
	"(Lorg/python/core/PyObject;Lorg/python/core/PyObject;)"
			"Lorg/python/core/PyException;");
    CHECK(jinit, return NULL, "Py.makeException failed");

    jobject pyexc = env->CallStaticObjectMethod(jcls, jinit, jtype, jvalue);
    CHECK(pyexc, return NULL, "Py.makeException(...) failed");

    return (jthrowable)pyexc;
}


//
// Py.makeException(Py.<type>, <msg>)
//
jthrowable PyException(JNIEnv *env, jobject jtype, const char *msg) {
    jobject pystr = NewPyString(env, msg);
    CHECK(pystr, return NULL, "");

    return PyException(env, jtype, pystr);
}




// Convenience function
void throwPyException(JNIEnv *env, jobject jtype, const char* msg) {
    jthrowable exc = PyException(env, jtype, msg);
    CHECK(exc, return, "PyException failed");
    env->Throw(exc);
}



// Convenience function
void throwFromErrno(JNIEnv *env, jobject jtype) {
    jobject jvalue = NewPyTuple(env, 2,
		NewPyInteger(env, errno),
		NewPyString(env, strerror(errno)));

    jthrowable exc = PyException(env, jtype, jvalue);
    CHECK(exc, return, "PyException failed");
    env->Throw(exc);
}



// Convenience function
void throwFromErrno(JNIEnv *env, jobject jtype, const char* msg) {
    jobject jvalue = NewPyTuple(env, 3,
		NewPyInteger(env, errno),
		NewPyString(env, strerror(errno)),
		NewPyString(env, msg));

    jthrowable exc = PyException(env, jtype, jvalue);
    CHECK(exc, return, "PyException failed");
    env->Throw(exc);
}




jobject NewPyTuple(JNIEnv *env, int count, ...) {
    jclass jobjcls = env->FindClass("org/python/core/PyObject");
    CHECK(jobjcls, return NULL, "FindClass(org/python/core/PyObject) failed");

    jobjectArray jarr = env->NewObjectArray(count, jobjcls, NULL);
    CHECK(jarr, return NULL, "new tuple jarr failed");

    va_list marker;
    va_start(marker, count);     /* Initialize variable arguments. */

    for (int i = 0; i < count; i++) {
	env->SetObjectArrayElement(jarr, i, va_arg(marker, jobject));
    }


    // Turn the array it into a tuple.
    // Corresponds to this java code: new PyTuple(jarr);

    jclass jpycls = env->FindClass("org/python/core/PyTuple");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jinit = env->GetMethodID(jpycls, "<init>", "([Lorg/python/core/PyObject;)V");
    CHECK(jinit, return NULL, "get PyTuple.<init> failed");

    jobject jpytup = env->NewObject(jpycls, jinit, jarr);
    CHECK(jpytup, return NULL, "new PyTuple(...) failed");

    return jpytup;
}




jobject NewPyDictionary(JNIEnv *env) {
    jclass jpycls = env->FindClass("org/python/core/PyDictionary");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/PyDictionary) failed");

    jmethodID jinit = env->GetMethodID(jpycls, "<init>", "()V");
    CHECK(jinit, return NULL, "get PyDictionary.<init> failed");

    jobject jpydict = env->NewObject(jpycls, jinit);
    CHECK(jpydict, return NULL, "new PyDictionary() failed");

    return jpydict;
}



int PyObject_setitem(JNIEnv *env, 
    jobject jpymap, jobject jkey, jobject jvalue) 
{
    jclass jobjcls = env->FindClass("org/python/core/PyObject");
    CHECK(jobjcls, return 0, "FindClass(org/python/core/PyObject) failed");

    jmethodID jsetitem = env->GetMethodID(jobjcls, "__setitem__",
		"(Lorg/python/core/PyObject;Lorg/python/core/PyObject;)V");
    CHECK(jsetitem, return 0, "get PyObject.__setitem__ failed");

    env->CallVoidMethod(jpymap, jsetitem, jkey, jvalue);
    return 1;
}



//
// new PyList()
//
jobject NewPyList(JNIEnv *env) {
    jclass jpycls = env->FindClass("org/python/core/PyList");
    CHECK(jpycls, return NULL, "FindClass(org/python/core/Py) failed");

    jmethodID jinit = env->GetMethodID(jpycls, "<init>", "()V");
    CHECK(jinit, return NULL, "get PyList.<init> failed");

    jobject jpylist = env->NewObject(jpycls, jinit);
    CHECK(jpylist, return NULL, "new PyList(...) failed");

    return jpylist;
}


int PyObject_invoke(JNIEnv *env, 
    jobject jobj, char* function, jobject jvalue) 
{
    jclass jobjcls = env->FindClass("org/python/core/PyObject");
    CHECK(jobjcls, return -1, "FindClass(org/python/core/PyObject) failed");

    jobject jfunction = env->NewStringUTF(function);
    CHECK(jfunction, return -1, "NewString(function) failed");

    jclass jstrcls = env->FindClass("java/lang/String");
    CHECK(jstrcls, return -1, "FindClass(java/lang/String) failed");

    jmethodID jintern = env->GetMethodID(jstrcls, "intern", "()Ljava/lang/String;");
    CHECK(jintern, return -1, "get String.intern() failed");

    jfunction = env->CallObjectMethod(jfunction, jintern);
    CHECK(jfunction, return -1, "jfunction.intern() failed");


    jmethodID jmeth = env->GetMethodID(jobjcls, "invoke",
	"(Ljava/lang/String;Lorg/python/core/PyObject;)Lorg/python/core/PyObject;");
    CHECK(jmeth, return -1, "get PyObject.method failed");

    env->CallObjectMethod(jobj, jmeth, jfunction, jvalue);
    return env->ExceptionOccurred() != NULL;
}




jclass findClass(JNIEnv *env, jclass *classp, jobject jobj) {
    jclass clazz = NULL;
    if (classp != NULL) {
	if (*classp == NULL) {
	    *classp = env->GetObjectClass(jobj);
	}
	clazz = *classp;
    } else {
	clazz = env->GetObjectClass(jobj);
    }
    return clazz;
}

jlong getLongInstanceField(JNIEnv* env,
    jclass *classp, jobject jobj, char* field)
{
    jclass clazz = findClass(env, classp, jobj);
    CHECK(clazz, return 0, "GetObjectClass failed");

    jfieldID fpId = env->GetFieldID(clazz, field, "J");
    CHECK(fpId, return 0, "GetFieldID");

    return env->GetLongField(jobj, fpId);
}

void setLongInstanceField(JNIEnv* env,
    jclass *classp, jobject jobj, char* field, jlong value)
{
    jclass clazz = findClass(env, classp, jobj);
    CHECK(clazz, return, "GetObjectClass failed");

    jfieldID fpId = env->GetFieldID(clazz, field, "J");
    CHECK(fpId, return, "GetFieldID");

    env->SetLongField(jobj, fpId, value);
}




jint getIntInstanceField(JNIEnv* env,
    jclass *classp, jobject jobj, char* field)
{
    jclass clazz = findClass(env, classp, jobj);
    CHECK(clazz, return 0, "GetObjectClass failed");

    jfieldID fpId = env->GetFieldID(clazz, field, "I");
    CHECK(fpId, return 0, "GetFieldID");

    return env->GetIntField(jobj, fpId);
}

void setIntInstanceField(JNIEnv* env,
    jclass *classp, jobject jobj, char* field, jint value)
{
    jclass clazz = findClass(env, classp, jobj);
    CHECK(clazz, return, "GetObjectClass failed");

    jfieldID fpId = env->GetFieldID(clazz, field, "I");
    CHECK(fpId, return, "GetFieldID");

    env->SetIntField(jobj, fpId, value);
}



jstring NewString(JNIEnv *env, char* value, int len) {
    jchar *jchars = (jchar*)malloc(sizeof(jchar) * len);
    for (int i = 0; i < len; i++)
	jchars[i] = value[i];

    jstring jstr = env->NewString(jchars, len);
    CHECK(jstr, return NULL, "get jstring failed");

    return jstr;
}


jstring StringConcat(JNIEnv *env, jstring s1, jstring s2) {
    int len1 = env->GetStringLength(s1);
    int len2 = env->GetStringLength(s2);

    jchar* jchars = (jchar*)malloc(sizeof(jchar) * (len1 + len2));
    jchar* jnewstr = jchars;

    jchar* value1 = (jchar*)env->GetStringChars(s1, NULL);
    int i;
    while (len1-- > 0)
	*jchars++ = *value1++;
    env->ReleaseStringChars(s1, value1);

    jchar* value2 = (jchar*)env->GetStringChars(s2, NULL);
    while (len2-- > 0)
	*jchars++ = *value2++;
    env->ReleaseStringChars(s2, value2);

    jstring jstr = env->NewString(jnewstr, len1 + len2);
    CHECK(jstr, return NULL, "get jstring failed");

    return jstr;
}

