
#include <jni.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

// A rather fragile check/print/return macro.
#define CHECK(var, ret, reason) \
    if (var == NULL) { \
	if (reason != NULL) \
	    printf("%s\n", reason); \
	ret; \
    } \



jobject NewPyInteger(JNIEnv *env, jint value);
jobject NewPyLong(JNIEnv *env, jlong value);
jobject NewPyString(JNIEnv *env, jstring value);
jobject NewPyString(JNIEnv *env, const char* value);
jobject NewPyFloat(JNIEnv *env, jdouble value);

jobject excType(JNIEnv* env, char* type);
jthrowable PyException(JNIEnv *env, jobject jtype, jobject jvalue);
jthrowable PyException(JNIEnv *env, jobject jtype, const char *msg);

void throwPyException(JNIEnv *env, jobject jtype, const char* msg);

void throwFromErrno(JNIEnv *env, jobject jtype);
void throwFromErrno(JNIEnv *env, jobject type, const char* msg);
jobject NewPyTuple(JNIEnv *env, int count, ...);
jobject NewPyDictionary(JNIEnv *env);
jobject NewPyList(JNIEnv *env);

int PyObject_setitem(JNIEnv *env, jobject jpymap, jobject jkey, jobject jvalue);
int PyObject_invoke(JNIEnv *env, jobject jobj, char* function, jobject jvalue);

jint getIntInstanceField(JNIEnv* env, jclass *classp, jobject jobj, char* field);
void setIntInstanceField(JNIEnv* env, jclass *classp, jobject jobj, char* field, jint v);

jlong getLongInstanceField(JNIEnv* env, jclass *classp, jobject jobj, char* field);
void setLongInstanceField(JNIEnv* env, jclass *classp, jobject jobj, char* field, jlong v);

jstring NewString(JNIEnv *env, char* value, int len);
jstring StringConcat(JNIEnv *env, jstring s1, jstring s2);

//
// This class is inspired by the LockedString found in java interface
// in bsddb www.sleepycat.com
//

class StringUTFChars
{
public:
    StringUTFChars(JNIEnv *env, jstring jstr) {
	env_ = env;
	jstr_ = jstr;
	if (jstr == 0)
            string = 0;
	else
            string = env->GetStringUTFChars(jstr, NULL);
    }

    ~StringUTFChars() {
	if (jstr_)
             env_->ReleaseStringUTFChars(jstr_, string);
    }

    int throwIfNull(jthrowable exc) {
	if (string == NULL)
	    env_->Throw(exc);
	return string == NULL;
    }

    operator const char*() {
	return string;
    }


private:
    const char *string;
    JNIEnv *env_;
    jstring jstr_;
};



class StringAsChars
{
public:
    StringAsChars(JNIEnv *env, jstring jstr) {
	env_ = env;
	jstr_ = jstr;
	if (jstr == 0)
            string = 0;
	else {
	    int len = env->GetStringLength(jstr);
            char *p = (char *)malloc(len);
	    string = p;
	    const jchar* chars = env->GetStringChars(jstr, NULL);
	    while (len-->0)
		*p++ = (char)*chars++;
	    env->ReleaseStringChars(jstr, chars);
	}
    }

    ~StringAsChars() {
	if (string)
             free((void*)string);
    }

    int throwIfNull(jthrowable exc) {
	if (string == NULL)
	    env_->Throw(exc);
	return string == NULL;
    }

    operator const char*() {
	return string;
    }


private:
    const char *string;
    JNIEnv *env_;
    jstring jstr_;
};


class IntArray
{
public:
    IntArray(JNIEnv *env, jintArray arr) {
	env_ = env;
	arr_ = arr;
	if (arr != 0) {
            count = env->GetArrayLength(arr);
	    array = env->GetIntArrayElements(arr, NULL);
	}
    }

    ~IntArray() {
	if (arr_ != 0)
	    env_->ReleaseIntArrayElements(arr_, array, 0);
    }


    jint& operator[](int idx) {
	return array[idx];
    }


    operator jint*() {
	return array;
    }


private:
    int count;
    jint *array;
    JNIEnv *env_;
    jintArray arr_;
};
